;;; tree-sitter-benchmark.el -*- lexical-binding: t; -*-

(require 'treesit)

;; Not compiled.
(let ((c-font-lock-settings-1
       `((c
          ,(with-temp-buffer
             (insert-file-contents-literally "./highlights.scm")
             ;; make capture names map to a face, any face
             (goto-char (point-min))
             (while (re-search-forward "@[a-z.]+" nil t)
               (replace-match "@font-lock-string-face" t))
             (buffer-substring (point-min) (point-max)))))))
  (with-temp-buffer
    (treesit-get-parser-create 'c)
    (setq-local treesit-font-lock-defaults
                '((c-font-lock-settings-1)))
    (font-lock-mode)
    (treesit-font-lock-enable)
    (insert-file-contents "xdisp.c")
    (let ((count 0))
      (apply #'message
             "Benchmark 3: fontify all of xdisp.c, 40 lines at a time.\
  took %2.2f, of which %2.2f is GC (%d gc runs), loop count: %s"
             (append
              (benchmark-run 1
                (while (/= (point-max) (point))
                  (font-lock-fontify-region (point) (line-end-position 40))
                  (forward-line 40)
                  (cl-incf count)))
              (list count))))))

;; Compiled.
(let ((c-font-lock-settings-1
       `((c
          ,(treesit-query-compile
            'c
            (with-temp-buffer
              (insert-file-contents-literally "./highlights.scm")
              ;; make capture names map to a face, any face
              (goto-char (point-min))
              (while (re-search-forward "@[a-z.]+" nil t)
                (replace-match "@font-lock-string-face" t))
              (buffer-substring (point-min) (point-max))))))))
  (with-temp-buffer
    (treesit-get-parser-create 'c)
    (setq-local treesit-font-lock-defaults
                '((c-font-lock-settings-1)))
    (font-lock-mode)
    (treesit-font-lock-enable)
    (insert-file-contents "xdisp.c")
    (let ((count 0))
      (apply #'message
             "Benchmark 3: fontify all of xdisp.c, 40 lines at a time.\
  took %2.2f, of which %2.2f is GC (%d gc runs), loop count: %s"
             (append
              (benchmark-run 1
                (while (/= (point-max) (point))
                  (font-lock-fontify-region (point) (line-end-position 40))
                  (forward-line 40)
                  (cl-incf count)))
              (list count))))))

;; Font-lock.
(with-temp-buffer
  (c-mode)
  (insert-file-contents "xdisp.c")
  (let ((count 0))
    (apply #'message
           "Benchmark 3: fontify all of xdisp.c, 40 lines at a time.\
  took %2.2f, of which %2.2f is GC (%d gc runs), loop count: %s"
           (append
            (benchmark-run 1
              (while (/= (point-max) (point))
                (font-lock-fontify-region (point) (line-end-position 40))
                (forward-line 40)
                (cl-incf count)))
            (list count)))))
